package parser;

import genvisitor.*;
/**
 * Factory for F1 grammar non-terminal objects.
 */
public class F1Fact extends ATVFactory {
    /**
     * Initializer lambda for this factory.
     */
    private ILambda _initializer = new ILambda() {
        public Object apply(Object param) {
            // change state to no-op
            _initializer = NoOpLambda.Singleton;

            // nothing to initialize
            return null;
        }
    };

    /**
     * Constructor for the F1 factory.
     *
     * @param tkz tokenizer to use
     */
    public F1Fact(ITokenizer tkz) {
        super(tkz);
    }

    private void addCmdTo(ATokVisitor<IGrammarToken, Object> target) {
      target.addCmd("Num", new AGenVisitor.IGenVisitorCmd<IGrammarToken, String, Object, AToken>() {
              public IGrammarToken apply(String idx, AToken host, Object... inp) {
                return new F1((NumToken)host);
              }
            });
    }
    
    /**
     * Make a token visitor to parse an E1a non-terminal.
     *
     * @return token visitor
     */
    public ATokVisitor<IGrammarToken, Object> makeVisitor() {
        initialize();
        // STUDENT TO COMPLETE
        return new ATokVisitor<IGrammarToken, Object>() {
          {
            addCmdTo(this);
          }
        };
    }

    /**
     * Make a token visitor that delegates to the given visitor in a chain of responsibility
     *
     * @param successor visitor to serve as successor in the chain
     */
    public ATokVisitor<IGrammarToken, Object> makeChainedVisitor(ATokVisitor<IGrammarToken, Object> successor) {
        initialize();
        // STUDENT TO COMPLETE
        return new ATokVisitor<IGrammarToken, Object>(successor) {
          {
            addCmdTo(this);
          }
        };
    }

    /**
     * Make the visitor.
     */
    private void initialize() {
        _initializer.apply(null);

    }
}

